@@ -5,7 +5,6 @@
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
-#include "split-double.h"
 #include "math.h"
 
 typedef struct bcd {
@@ -18,7 +17,7 @@ bcd_t;
 
 bcd_t bcddata[0x800] = {
 {0, 0, 0, 0}, /* -inf */
-#line 22 "bcd.c"
+#line 21 "bcd.c"
 {-1022, -308, 0x22250738, 0x58507201},
 {-1021, -308, 0x44501477, 0x17014403},
 {-1020, -308, 0x89002954, 0x34028806},
@@ -2065,17 +2064,18 @@ bcd_t bcddata[0x800] = {
 {1021, 307, 0x22471164, 0x18577895},
 {1022, 307, 0x44942328, 0x37155789},
 {1023, 307, 0x89884656, 0x74311579},
-#line 23 "bcd-c.tmpl"
+#line 22 "bcd-c.tmpl"
 {0, 0, 0, 0} /* +inf */
 };
 
 #define BYTE(bcd,j) (((bcd) >> ((7-j)*4)) & 0xf)
 
-int bit(unsigned char * z, int b)
+int bit (uint64_t * bits, unsigned int b)
 {
-    int byte = b / 8;
-    int bit = b % 8;
-    if (z[byte] & 1<<bit) {
+    uint64_t bit;
+    bit = 1ull<<b;
+    //printf ("%lld\n", bit);
+    if ((*bits) & bit) {
 	return 1;
     }
     return 0;
@@ -2084,39 +2084,44 @@ int bit(unsigned char * z, int b)
 int
 print_double (double d, char * buffer)
 {
-    split_double_t * sp;
-    unsigned char * z;
     int bytes[16] = {0};
     int powten;
     int i;
     int exp;
     double check = 0.0;
     int j;
+// The number as bits
+    uint64_t * bits;
     // Current print point within buffer.
     char * p;
     // Number of bytes printed.
     int printed;
+    // The sign of d.
+    unsigned int sign;
 
     p = buffer;
     printed = 0;
 
-    sp = (split_double_t *) & d;
-    z = (unsigned char *) & d;
-    exp = sp->exp;
-    powten = bcddata[sp->exp].exp;
-//    printf ("exp = %d, pow ten = %d\n", sp->exp, powten);
-#if 0
-    printf ("data = %08X;%08X x 10^{%d}\n", bcddata[sp->exp].upper,
-	    bcddata[sp->exp].lower, bcddata[sp->exp].exp);
+    bits = (uint64_t *) & d;
+    exp = ((*bits) >> 52) & 0x7ff;
+    sign = (*bits) >> 63;
+    powten = bcddata[exp].exp;
+#ifdef DEBUG
+    printf ("Input is %g\n", d);
+    printf ("Decomposed exp = %d, pow ten = %d\n", exp, powten);
+    printf ("fraction as hex: %llX\n", (*bits) & 0x1fffffffffffffll);
+    printf ("data = %08X;%08X x 10^{%d}\n", bcddata[exp].upper,
+	    bcddata[exp].lower, bcddata[exp].exp);
 #endif
     for (j = 0; j < 8; j++) {
-	bytes[j] = BYTE(bcddata[sp->exp].upper, j);
-	bytes[j + 8] = BYTE(bcddata[sp->exp].lower, j);
+	bytes[j] = BYTE(bcddata[exp].upper, j);
+	bytes[j + 8] = BYTE(bcddata[exp].lower, j);
     }
     check = pow (2.0, exp - 1023);
-//    printf ("check = %g\n", check);
-//    exit (0);
-    //printf ("%llX\n", sp->fraction);
+
+#ifdef DEBUG
+    printf ("check = %g\n", check);
+#endif
     for (i = 1; i <= 52; i++) {
 	bcd_t shift;
 	int powtenshift;
@@ -2125,18 +2130,18 @@ print_double (double d, char * buffer)
 	if (expi < 0) {
 	    break;
 	}
-	if (bit (z, 52 - i)) {
+	if (bit (bits, 52 - i)) {
 	    int j;
 	    int maxupper;
 	    int maxlower;
-#if 0
-	    printf ("data = %08X;%08X x 10^{%d}\n", bcddata[expi].upper,
+#ifdef DEBUG
+	    printf ("a bit %d: data = %08X;%08X x 10^{%d}\n", i, bcddata[expi].upper,
 		    bcddata[expi].lower, bcddata[expi].exp);
 #endif
 	    check += pow (2.0, expi - 1023);
 	    shift = bcddata[expi];
 	    powtenshift = powten - shift.exp;
-#if 0
+#ifdef DEBUG
 	    printf ("bit = %d exp = %d powtenshift = %d check=%.9g\n", i, exp - i, powtenshift, check);
 #endif 
 	    maxupper = 8;
@@ -2151,7 +2156,7 @@ print_double (double d, char * buffer)
 	    }
 	    for (j = 0; j < maxupper; j++) {
 		int digit = (shift.upper >> ((7-j)*4)) & 0xf;
-#if 0
+#ifdef DEBUG
 		if (digit) {
 		    printf ("Adding digit %d\n", digit);
 		}
@@ -2160,15 +2165,15 @@ print_double (double d, char * buffer)
 	    }
 	    for (j = 0; j < maxlower; j++) {
 		int digit = (shift.lower >> ((7-j)*4)) & 0xf;
-#if 0
+#ifdef DEBUG
 		if (digit) {
 		    printf ("Adding digit %d\n", digit);
 		}
 #endif
 		bytes[powtenshift + 8 + j] += digit;
 	    }
+#ifdef DEBUG
 	    int k;
-#if 0
 	    for (k = 0; k < 15; k++) {
 		printf ("%d ", bytes[k]);
 	    }
@@ -2176,12 +2181,21 @@ print_double (double d, char * buffer)
 #endif /* 0 */
 	}
     }
-    for (i = 15; i >= 0; i--) {
-	if (bytes[i] > 10) {
+    for (i = 15; i > 0; i--) {
+	if (bytes[i] >= 10) {
+//	    printf ("%d\n", bytes[i]);
 	    bytes[i - 1] += bytes[i] / 10;
 	    bytes[i] %= 10;
 	}
     }
+    if (bytes[0] >= 10) {
+	powten += 1;
+	for (i = 14; i > 0; i--) {
+	    bytes[i+1]=bytes[i];
+	}
+	bytes[1] = bytes[0] % 10;
+	bytes[0] /= 10;
+    }
 #define MAXDIGITS 14
     // Round off, maximum byte is byte 14.
     if (bytes[MAXDIGITS] >= 5) {
@@ -2199,8 +2213,12 @@ print_double (double d, char * buffer)
 		 __FILE__, __LINE__, printed);
 	return -1;
     }
+    if (sign) {
+    printed += sprintf (p, "-");
+    p = buffer + printed;
+}
     printed += sprintf (p, "%d.", bytes[0]);
-	p = buffer + printed;
+    p = buffer + printed;
     for (i = 1; i < MAXDIGITS; i++) {
 #if 0
 	if (i > 1 && (i - 1) % 3 == 0) {
@@ -2247,16 +2265,20 @@ static void doit (double d)
 {
     char buffer[0x100] = {0};
     print_double (d, buffer);
-    printf ("%.13e\n%s\n", d, buffer);
+    printf ("expec:\t%.13e\ngot:\t%s\n", d, buffer);
 //    ruler ();
 }
 
 int
 main (int argc, char ** argv)
 {
-    char buffer[0x100] = {0};
     double d;
 
+    d = 10.0;
+    doit (d);
+    d = 11.0;
+    doit (d);
+//    exit (0);
     d = 1234.5678;
     doit (d);
     d = 9.784536e100;
